/*****************************************************************************
 Base template for vector owner of pointers, it destroys all owned elements
 before efectively remove them from the list

 Classes defined:
   CDimListIO
   CDimVectorOwner
   CDimDequeOwner

 IMPLEMENTATION

 Author: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>
 Copyright (c) 2006 Vision Research Lab, UCSB <http://vision.ece.ucsb.edu>

 History:
   04/19/2006 16:20 - First creation

 Ver : 1
*****************************************************************************/

#include "xvectors.h"

//----------------------------------------------------------------------------
// CDimListIO
//----------------------------------------------------------------------------

int CDimListIO::fromFile(const std::string *fileName) {
  std::ifstream f (fileName->c_str(), std::ios::in|std::ios::binary);
  if (!f.is_open()) return -1;

  readHeader( &f );
  int size = fromFile( &f );
  f.close();
  return size;
}

int CDimListIO::toFile(const std::string *fileName) {
  std::ofstream f (fileName->c_str(), std::ios::out|std::ios::binary);
  if (!f.is_open()) return -1;

  writeHeader( &f );
  int size = toFile( &f );
  f.close();
  return size;
}

int CDimListIO::fromFile(const char* fileName) {
  std::string fn = fileName;
  return fromFile(&fn);
}

int CDimListIO::toFile(const char* fileName) {
  std::string fn = fileName;
  return toFile(&fn);
}
