/*****************************************************************************
  descriptors for textures, tiles and images - ASCII format

 IMPLEMENTATION

 Author: Dima V. Fedorov <mailto:dima@dimin.net> <http://www.dimin.net/>
 Copyright (c) 2006 Vision Research Lab, UCSB <http://vision.ece.ucsb.edu>

 History:
   04/19/2006 16:20 - First creation

 Ver : 1
*****************************************************************************/

#include "textureDescriptorsTxt.h"

#include <string>
#include <vector>
#include <fstream>
#include <cmath>

inline int iround(double x) {
  return (int) floor(x + 0.5);
}


//----------------------------------------------------------------------------
// TDimTileDescriptorTxt
//----------------------------------------------------------------------------

CDimTileDescriptorTxt::CDimTileDescriptorTxt()
  : CDimTileDescriptor() 
{
  memset( tile_pos_rowcol, 0, sizeof(unsigned int)*2 );
}

int CDimTileDescriptorTxt::fromFile(std::ifstream *fs1, std::ifstream *fs2) {
  if (fs1 == NULL) return -1;
  if (fs2 == NULL) return -1;
  char line1[2048] = "\0";
  char name[2048] = "\0";

// first file
//D:\ITR\Mcode\ONLClassification_adj\testImages\NZ103_N.tif	0	0	768	512	32	64
//8.138095  3.641947  3.350249  5.133218  3.025223  3.579220  8.553727  3.951319  3.806584  5.080593  4.368159  4.339326  9.549284  5.370177  5.107850  5.611490  5.354511  5.236495  10.207483  6.522279  5.088810  4.969350  5.253484  6.293095  9.557642  5.652368  5.341338  5.586662  5.227682  5.544317  10.860653  5.509316  4.857845  6.997334  4.468487  5.454245  11.548512  6.211828  5.985483  7.385754  6.881027  6.568757  12.957651  8.171101  8.174067  8.650476  8.321845  7.990706  14.014027  10.017037  8.672212  8.542879  8.754088  9.753015  13.375184  9.476869  9.447037  9.545653  9.326786  9.358844 

  fs1->getline (line1, 2048 );
  if (fs1->fail()) return -1;

#ifdef HAVE_SECURE_C  
  sscanf_s(line1, "%s %d %d %d %d %d  %d", name, 2048, &tile_pos_rowcol[0], &tile_pos_rowcol[1],
    &image_size[0], &image_size[1], &tile_step, &tile_size_pixels );
#else
  sscanf(line1, "%s %d %d %d %d %d  %d", name, &tile_pos_rowcol[0], &tile_pos_rowcol[1],
    &image_size[0], &image_size[1], &tile_step, &tile_size_pixels );
#endif

  // extract the the file name
  char *l = strrchr(name, '\\');
  if (l != NULL) {
    image_name = ++l;
  }

  // read textures
  fs1->getline (line1, 2048 );
  if (fs1->fail()) return -1;

  l = line1;
  int i;
  float t;
  for (i=0; i<TEXTURE_DIMS; ++i) {
#ifdef HAVE_SECURE_C  
    sscanf_s(l, "%f", &t);
#else
    sscanf(l, "%f", &t);
#endif
    texture[i] = t;
    l = strchr(l, ' ');
    l+=2;
  }

/* second file
imgID X Y Z
1 0 0 0 
*/
  fs2->getline (line1, 2048 );
  if (fs2->fail()) return -1;
#ifdef HAVE_SECURE_C  
  sscanf_s(line1, "%d %d %d %d", &image_id, &position_in_img[0], &position_in_img[1], &position_in_img[2] );
#else
  sscanf(line1, "%d %d %d %d", &image_id, &position_in_img[0], &position_in_img[1], &position_in_img[2] );
#endif    
  return 1;
}


//----------------------------------------------------------------------------
// ground truth
//----------------------------------------------------------------------------

int CDimGTDescriptorTxt::fromFile(std::ifstream *fs) {

  if (fs == NULL) return -1;
  char line1[2048];
  float fn[12];

// first file
//imgID X Y 1. BG 2.ILM 3.GCL 4.IPL 5.INL 6.OPL 7.ONL 8.IS 9.OS 10.RPE 11.M/L
//1 0 0 0.65 0.00 0.35 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00

  fs->getline (line1, 2048 );
  if (fs->fail()) return -1;

#ifdef HAVE_SECURE_C  
  sscanf_s(line1, "%d %d %d %f %f %f %f %f %f %f %f %f %f %f", &image_id, &tile_pos_rowcol[0], &tile_pos_rowcol[1],
    fn+0, fn+1, fn+2, fn+3, fn+4, fn+5, fn+6, fn+7, fn+8, fn+9, fn+10 );
#else
  sscanf(line1, "%d %d %d %f %f %f %f %f %f %f %f %f %f %f", &image_id, &tile_pos_rowcol[0], &tile_pos_rowcol[1],
    fn+0, fn+1, fn+2, fn+3, fn+4, fn+5, fn+6, fn+7, fn+8, fn+9, fn+10 );
#endif

  int i;
  for (i=0; i<12; ++i) {
    layers_percents[i] = iround(fn[i]*100.0);
    layers_gt[i] = (unsigned char) layers_percents[i];
  }
 
  return 1;
}


//----------------------------------------------------------------------------
// CDimImageItemTxt
//----------------------------------------------------------------------------

int CDimImageItemTxt::fromFile(std::ifstream *fs)
{
  if (fs == NULL) return -1;
  char line[2048] = "\0";
  char name[2048] = "\0";
  float mag[2];

  fs->getline (line, 2048 );
  if (fs->fail()) return -1;

#ifdef HAVE_SECURE_C  
  sscanf_s(line, "%d\t%s\t%f\t%f", &image_id, name, 2048, &mag[0], &mag[1]);
#else
  sscanf(line, "%d\t%s\t%f\t%f", &image_id, name, &mag[0], &mag[1]);
#endif   
  image_name = name;
  pixel_size[0] = mag[0];
  pixel_size[1] = mag[0];
  pixel_size[2] = mag[1];

  return 1;
}

//----------------------------------------------------------------------------
// CDimTileListTxt
//----------------------------------------------------------------------------

int CDimTileListTxt::fromFile(const char *fileName1, const char *fileName2)
{
  std::string fn1 = fileName1;
  std::string fn2 = fileName2;
  return fromFile(&fn1, &fn2);
}

int CDimTileListTxt::fromFile(const std::string *fileName1, const std::string *fileName2)
{
  std::ifstream f1 (fileName1->c_str(), std::ios::in);
  if (!f1.is_open()) return -1;
  std::ifstream f2 (fileName2->c_str(), std::ios::in);
  if (!f2.is_open()) return -1;

  CDimTileDescriptorTxt td;
  this->clear();
  while ( (td.fromFile(&f1, &f2)>0) || (!f1.eof() && !f2.eof()) )
  {
    this->add( td );
  }
  
  f1.close();
  f2.close();
  return (int) this->size();
}

int CDimTileListTxt::findImageById(unsigned int imID) {
  int res = -1;
  unsigned int i;
  for (i=0; i<this->size(); ++i)
    if (this->at(i)->image_id == (int) imID) {
      res = i;
      break;
    }
  return res;
}

int CDimTileListTxt::findMaxZTileByImageId(unsigned int imID) {
  int res = findImageById(imID);
  if (res == -1) return res;
  unsigned int i, zmax=0;

  for (i=res; i<this->size(); ++i) {
    if (this->at(i)->image_id != (int) imID) break;
    if (zmax < this->at(i)->position_in_img[2]) {
      res = i;
    }
  }
  return res;
}





